package Plugins::LightsOut::Plugin;


#
# Integrate Squeezecenter with Windows Home Server Add-In LightsOut 
#
# Details about Lights Out can be found here:
# http://www.home-server-blog.de/add-ins/strom-und-laufzeitverwaltung/lightsout/
#
#
# Thanks to Gordon Harris and the other authors of the PreventStandby Plugin
#
# This Plugin uses all the original PreventStandby Code to modify the registry to interface with LightsOut
#
# Original idea and discussion was here in posts 704ff. :
# http://forums.slimdevices.com/showthread.php?t=48521
#
#
# -> History
#
# 0.1  - 2009-06-10 - First Release
#
			   

use strict;
use Win32::API;
use Win32::TieRegistry;

use Slim::Utils::Log;
use Slim::Utils::Prefs;
use Slim::Utils::OSDetect;

if ( !main::SLIM_SERVICE && !$::noweb ) {
	require Plugins::LightsOut::Settings;
}


# How many seconds between checks for busy clients..
# Reduce this value when testing, unless you are very patient.
use constant INTERVAL => 60;

# time() we last checked for client activity
my $lastchecktime = time;

# Number of intervals that the cliets have been idle.
my $hasbeenidle = 0;

my $key;
$Registry->Delimiter("/");

my $prefs = preferences('plugin.lightsout');


$prefs->migrate(1, sub {
	$prefs->set('idletime', Slim::Utils::Prefs::OldPrefs->get('idletime') || 20);
	$prefs->set('checkpower', Slim::Utils::Prefs::OldPrefs->get('checkpower') || 0);
	$prefs->set('lomode', Slim::Utils::Prefs::OldPrefs->get('lomode') || 2);
	1;
});

$prefs->setValidate({ 'validator' => 'intlimit', 'low' => 0, 'high' => 240 }, 'idletime');
$prefs->setChange(\&idletime_change, 'idletime');
$prefs->setChange(\&checkpower_change, 'checkpower');
$prefs->setChange(\&lomode_change, 'lomode');

 

my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.lightsout',
	'defaultLevel' => 'ERROR',
	'description'  => getDisplayName(),
});

sub lightsout_on {
	$key=$Registry->{"LMachine/SOFTWARE/AxoNet Software GmbH/LightsOut/"};
	$key->{"/Action"} = [ pack("L",$prefs->get('lomode') ), "REG_DWORD" ];
}

sub lightsout_off {
	$key=$Registry->{"LMachine/SOFTWARE/AxoNet Software GmbH/LightsOut/"};
	$key->{"/Action"} = [ pack("L",0), "REG_DWORD" ];  
}

sub getDisplayName {
	return 'PLUGIN_LIGHTSOUT';
}


sub initPlugin {
	


	if ( !main::SLIM_SERVICE && !$::noweb ) {
		Plugins::LightsOut::Settings->new;
	}

	if (my $idletime = $prefs->get('idletime')) {
		$log->debug("LightsOut active after $idletime minutes of player idle time.")
		
		
	}
	else {
		$log->debug("LightsOut now deactivated.")
		
		
	}

	checkClientActivity();
}


sub checkClientActivity {
	my $currenttime = time();

	# Reset the idle countdown counter if 1). scanning, 2). firmware updating or playing, or
	# 3). time-shift (i.e. DST system time change) or we've resumed from standby or hibernation..
	my $idletime = $prefs->get('idletime');
	
	if ($idletime) {
		
		if ( Slim::Music::Import->stillScanning() || _hasResumed($currenttime) || _playersBusy() ) {
			
			$hasbeenidle = 0;
			main::DEBUGLOG && $log->is_debug && $log->debug("Resetting idle counter.    " . ($idletime - $hasbeenidle) . " minutes left in allowed idle period.");
		}
		
		else {
			
			$hasbeenidle++;
			
			if ($hasbeenidle < $idletime) {
				main::DEBUGLOG && $log->is_debug && $log->debug("Incrementing idle counter. " . ($idletime - $hasbeenidle) . " minutes left in allowed idle period.");
			}
		}
	}

	

	# If idletime is set to zero in settings, ALWAYS prevent standby..
	# Otherwise, only prevent standby if we're still in the idle time-out period..
	if ( (!$idletime) || $hasbeenidle < $idletime) {
		
		main::INFOLOG && $log->is_info && $log->info("Preventing Lightsout to Shut System Down...");
		lightsout_off();	
	}
	
	else {
		main::INFOLOG && $log->is_info && $log->info("Players have been idle for $hasbeenidle minutes. Allowing LightsOut to do it's magic...");
		lightsout_on();
	}

	$lastchecktime = $currenttime;
	
	Slim::Utils::Timers::killTimers( undef, \&checkClientActivity );
	Slim::Utils::Timers::setTimer(
		undef, 
		time + INTERVAL, 
		\&checkClientActivity
	);

	return 1;
}

sub _playersBusy {
	
	my $checkpower = $prefs->get('checkpower');
	
	for my $client (Slim::Player::Client::clients()) {
		
		if ($checkpower && $client->power()) {
			main::DEBUGLOG && $log->is_debug && $log->debug("Player " . $client->name() . " is powered " . ($client->power() ? "on" : "off") . "...");
			return 1;
		}
		
		if ( $client->isUpgrading() || $client->isPlaying() ) {
			main::DEBUGLOG && $log->is_debug && $log->debug("Player " . $client->name() . " is busy...");
			return 1;
		}
	}
	return 0;
}

sub _hasResumed {
	my $currenttime = shift;

	# We've resumed if the current time is more than two minutes later than the last check time, or
	# if the current time is earlier than the last check time (system time change)
	
	if ( $currenttime > ($lastchecktime + (INTERVAL * 2)) || $currenttime < $lastchecktime ) {
		
		main::DEBUGLOG && $log->debug("System has resumed...");
		return 1;
	}
	
	return 0;
}


sub idletime_change {
	my ($pref, $value) = @_;
	
	$value ||= 0;

	main::DEBUGLOG && $log->debug("Pref $pref changed to $value. Resetting idle counter.");

	# Reset our counter on prefs change..
	$hasbeenidle = 0;

	if ($value) {
		$log->debug("LightsOut activated after $value minutes of player idle time.")
	} else {
		$log->debug("LightsOut disabled.")
	}
}

sub checkpower_change {
	my ($pref, $value) = @_;
	
	$value ||= 0;

	main::DEBUGLOG && $log->debug("Pref $pref changed to $value. Resetting idle counter.");

	# Reset our counter on prefs change..
	$hasbeenidle = 0;

	if ($value) {
		$log->debug("LightsOut active when players are powered on.")
	}
}


sub lomode_change {
	my ($pref, $value) = @_;
	
	$value ||= 0;

	main::DEBUGLOG && $log->debug("Pref $pref changed to $value. Resetting idle counter.");

	# Reset our counter on prefs change..
	$hasbeenidle = 0;

	if ($value) {
		$log->debug("Changed Lights Out Mode to $value.")
	}
}


sub shutdownPlugin {
		
	lightsout_on();
		
	Slim::Utils::Timers::killTimers( undef, \&checkClientActivity );
	
}

1;

__END__
